import contextlib
import os
from collections.abc import Generator
from pathlib import Path
from typing import Optional

@contextlib.contextmanager
def chdir_context(path: Optional[Path]) -> Generator[None]:
    """
    Non parallel-safe context manager to change the current working directory. If `path` is `None`, it does nothing. If `path` is not None, it changes the current working directory upon entering and restores the old one on exit.

    Unlike `contextlib.chdir()`, this context manager is NOT reentrant or reusable.

    See `contextlib.chdir()`'s documentation on notes about threaded and async contexts.
    """

    if path is None:
        yield None
    else:
        try:
            old_cwd = Path.cwd()
            os.chdir(path)
            yield None
        finally:
            os.chdir(old_cwd)



__all__ = [
    'temporary_directory',
]

import contextlib
import sys
import tempfile
from collections.abc import Generator
from pathlib import Path
from typing import Optional, TypedDict

if sys.version_info >= (3, 12):
    class DeleteKeyword(TypedDict, total=False):
        delete: bool
else:
    class DeleteKeyword(TypedDict, total=False):
        pass

@contextlib.contextmanager
def temporary_directory(
    suffix: Optional[str] = None,
    prefix: Optional[str] = None,
    dir: Optional[Path] = None,
    ignore_cleanup_errors: bool = False,
    delete: Optional[bool] = None,
) -> Generator[Path]:
    """
    Context manager similar to `tempfile.TemporaryDirectory` except it returns the created directory's name as an absolute `pathlib.Path` instead of `str`.

    Supports all arguments of `tempfile.TemporaryDirectory`, including `delete` introduced in Python 3.12. If `delete` is specified on Python 3.11 or older, ValueError is raised.

    Note that the returned path is always absolute, even if the `dir` parameter is relative. This is consistent with how `tempfile.TemporaryDirectory` works starting with Python 3.12.
    """

    delete_kwargs: DeleteKeyword = dict()
    if delete is not None:
        if sys.version_info >= (3, 12):
            delete_kwargs['delete'] = delete
        else:
            raise ValueError('tempfile.TemporaryDirectory() only supports the "delete" argument from Python 3.12')

    with tempfile.TemporaryDirectory(suffix, prefix, dir, ignore_cleanup_errors, **delete_kwargs) as tmp_dir:
        yield Path(tmp_dir).absolute()



__all__ = [
    'split_path_list',
    'join_path_list',
    'deduplicate_path_list',
]

import os
from pathlib import Path
from typing import Sequence

import more_itertools

def split_path_list(path_string: str) -> list[Path]:
    """
    Split `PATH` string based on `os.pathsep` and convert each component to `pathlib.Path`.

    Empty components will be removed, i.e. leading, trailing or duplicated separators will not cause issues.

    In contrast to `str.split()`, if the string is empty, the function will return an empty list.
    """

    if path_string=='':
        return []

    return [Path(d) for d in path_string.split(os.pathsep) if d]

def join_path_list(path_list: Sequence[Path | str | None]) -> str:
    """
    Create `PATH` string (`os.pathsep`-separated string) from list of paths. The list is allowed to contain `Path` objects, strings and even `None`. Empty strings and `None`s will be removed before joining the list.
    """

    return os.pathsep.join(str(d) for d in path_list if d)

def deduplicate_path_list(path_list: Sequence[Path | str | None]) -> list[Path]:
    """
    Remove duplicates from path list, keeping only the first occurence. The list is allowed to contain `Path` objects, strings and even `None`. Empty strings and `None`s will be removed before deduplication, and the rest is converted to `Path`.
    """

    # Note that it's important to keep the relative order of directories in the PATH list, we simply want to remove
    # duplicates after the first instance.
    return list(more_itertools.unique_everseen(Path(d) for d in path_list if d))
